@@ -24,6 +24,60 @@
 
 
 #include "mz.h"
+#include "cli.h"
+
+
+#define MZ_CDP_HELP \
+   		"| CDP type: Send arbitrary CDP packets.\n" \
+		"| Note:\n" \
+		"| - The Ethernet dst and src MAC addresses can be specified but can be also 'rand'.\n" \
+		"| - If dst and src are NOT specified then practical defaults are used (src=own MAC, dst=01:00:0C:CC:CC:CC).\n" \
+		"|\n" \
+		"|  ARGUMENT SYNTAX: -t cdp [arguments]\n" \
+		"|\n" \
+		"|  ARGUMENTS:\n" \
+		"|\n" \
+		"|    version ...... 0-255, default: 2\n" \
+		"|    ttl     ...... 0-255, default: 180 s\n" \
+		"|    sum     ...... 0000-ffff, default: automatically computed\n" \
+		"|\n" \
+		"|   TLVs:                  Description:                           Example:\n" \
+		"|\n" \
+		"|    tlv_id        ....... Device ID                              Mausezahn station\n" \
+		"|    tlv_address   ....... Sending interface address              10.1.1.2\n" \
+		"|    tlv_portid    ....... Port Identifier                        2/23\n" \
+		"|    tlv_cap       ....... Capabilities (hex<7f)                  2a\n" \
+		"|    tlv_version   ....... Software Version                       ver3.0\n" \
+		"|    tlv_platform  ....... Hardware Platform                      WS-C6509-E\n" \
+		"|    tlv_vtpdomain ....... VTP Management Domain                  MyVTPdomain\n" \
+		"|    tlv_native    ....... Native VLAN number (0-4095)            42\n" \
+		"|    tlv_duplex    ....... Full or half duplex                    full\n" \
+		"|    tlv_mgmt      ....... Management IP address                  192.168.1.2\n" \
+		"|\n" \
+		"|    tlv        .......... Create ANY TLV using the format: tlv=<type>/<value>, such as tlv=42/mausezahn\n" \
+		"|                    Note: Currently you must omit spaces within <value>! Use underscore instead.\n" \
+		"|    tlvhex     .......... Create ANY TLV and specify the value in hexformat, such as tlv=42/ca:fe:ba:be\n" \
+		"|    payload|p  .......... Optional additional TLVs or any other bytes specified in hex\n" \
+		"|\n" \
+		"|    When the tlv* arguments are used, the TLV length parameter is automatically set.\n" \
+		"|\n" \
+		"|    The capability flags from MSB to LSB are:\n" \
+		"|      0 - Repeater - IGMP - Host - Switch - SrcRouteBrdg - TranspBrdg - Router\n" \
+		"|\n" \
+		"|    Optionally the keyword 'change' will create a different System name TLV every time a CDP\n" \
+		"|    packet is sent. This can be used to fill up a CDP database with different test values.\n" \
+		"|    Additionally use the '-a rand' command to use different source MAC addresses.\n" \
+		"|\n" \
+		"| EXAMPLES:\n" \
+		"|\n" \
+		"|    Announce Device ID 'Espresso3000', Capabilities: Router, native VLAN 301:\n" \
+		"|    mz eth0 -t cdp \"tlv_id=Espresso3000, tlv_cap=01, tlv_native=301\"\n" \
+		"|\n" \
+		"|    Create another TLV using the payload interface (here voice VLAN 400):\n" \
+		"|    mz eth0 -t cdp p=00:0e:00:07:01:01:90\n" 
+
+		
+		
 
 
 u_int16_t checksum16 (u_int16_t len, u_int8_t buff[])
@@ -168,58 +222,19 @@ int send_cdp ()
 
    if (getarg(tx.arg_string,"help", NULL)==1)
      {
-	fprintf(stderr,"\n"
-		MAUSEZAHN_VERSION
-		"\n"
-		"| CDP type: Send arbitrary CDP packets.\n"
-		"| Note:\n"
-		"| - The Ethernet dst and src MAC addresses can be specified but can be also 'rand'.\n"
-		"| - If dst and src are NOT specified then practical defaults are used (src=own MAC, dst=01:00:0C:CC:CC:CC).\n"
-		"|\n"	  
-		"|  ARGUMENT SYNTAX: -t cdp [arguments]\n"
-		"|\n"
-		"|  ARGUMENTS:\n"
-		"|\n"
-		"|    version ...... 0-255, default: 2\n"
-		"|    ttl     ...... 0-255, default: 180 s\n"
-		"|    sum     ...... 0000-ffff, default: automatically computed\n"
-		"|\n"
-		"|   TLVs:                  Description:                           Example:\n"   
-		"|\n"
-		"|    tlv_id        ....... Device ID                              Mausezahn station\n"
-		"|    tlv_address   ....... Sending interface address              10.1.1.2\n"
-		"|    tlv_portid    ....... Port Identifier                        2/23\n"
-		"|    tlv_cap       ....... Capabilities (hex<7f)                  2a\n"
-		"|    tlv_version   ....... Software Version                       ver3.0\n"
-		"|    tlv_platform  ....... Hardware Platform                      WS-C6509-E\n"
-		"|    tlv_vtpdomain ....... VTP Management Domain                  MyVTPdomain\n"
-		"|    tlv_native    ....... Native VLAN number (0-4095)            42\n"
-		"|    tlv_duplex    ....... Full or half duplex                    full\n"
-		"|    tlv_mgmt      ....... Management IP address                  192.168.1.2\n"
-		"|\n"
-		"|    tlv        .......... Create ANY TLV using the format: tlv=<type>/<value>, such as tlv=42/mausezahn\n"
-		"|                    Note: Currently you must omit spaces within <value>! Use underscore instead.\n"
-		"|    tlvhex     .......... Create ANY TLV and specify the value in hexformat, such as tlv=42/ca:fe:ba:be\n" 
-		"|    payload|p  .......... Optional additional TLVs or any other bytes specified in hex\n"
-		"|\n"
-		"|    When the tlv* arguments are used, the TLV length parameter is automatically set.\n"
-		"|\n"
-		"|    The capability flags from MSB to LSB are:\n"
-		"|      0 - Repeater - IGMP - Host - Switch - SrcRouteBrdg - TranspBrdg - Router\n"
-		"|\n"
-		"|    Optionally the keyword 'change' will create a different System name TLV every time a CDP\n"
-		"|    packet is sent. This can be used to fill up a CDP database with different test values.\n"
-		"|    Additionally use the '-a rand' command to use different source MAC addresses.\n"
-		"|\n"
-		"| EXAMPLES:\n"
-		"|\n"
-		"|    Announce Device ID 'Espresso3000', Capabilities: Router, native VLAN 301:\n"
-		"|    mz eth0 -t cdp \"tlv_id=Espresso3000, tlv_cap=01, tlv_native=301\"\n"
-		"|\n"
-		"|    Create another TLV using the payload interface (here voice VLAN 400):\n"
-		"|    mz eth0 -t cdp p=00:0e:00:07:01:01:90\n"
-		"\n");
-	exit(0);
+	if (mz_port)
+	  {
+	     cli_print(gcli, "%s", MZ_CDP_HELP);
+	     return -1;
+	  }
+	else
+	  {
+	     fprintf(stderr,"\n" 
+		     MAUSEZAHN_VERSION
+		     "\n%s", MZ_CDP_HELP);
+	     exit(0);
+	  }
+	
      }
 
    ///////////////////////////////////////////////////////////////////////
@@ -377,7 +392,7 @@ int send_cdp ()
 	  }
 	else
 	  {
-	     str2hex(argval, value+3);
+	     str2hex(argval, value+3, 1020);
 	     if (value[3]>0x7f)
 	       {
 		  fprintf(stderr," mz/send_cdp: Capability value must not exceed 7F(hex)\n");
@@ -522,7 +537,7 @@ int send_cdp ()
      {
 	// split in TYPE and VALUE
 	sscanf(argval, "%u/%s",  &type2, pld);
-	len2 = str2hex(pld, value2);
+	len2 = str2hex(pld, value2, 1023);
      }
 
    
@@ -531,7 +546,7 @@ int send_cdp ()
    // 
    if ( (getarg(tx.arg_string,"payload", argval)==1) || (getarg(tx.arg_string,"p", argval)==1))
      {
-	len = str2hex (argval, value);
+	len = str2hex (argval, value, 1023);
 	memcpy((void*) tx.cdp_payload+next_pbyte, (void*) value, len);
 	next_pbyte += len;
      }
@@ -580,7 +595,7 @@ int send_cdp ()
    
    if (check_eth_mac_txt(ETH_DST))  // if '1' then user did not set MAC address (or problem occurred)
      {
-	str2hex("01:00:0C:CC:CC:CC", tx.eth_dst);
+	str2hex("01:00:0C:CC:CC:CC", tx.eth_dst, 6);
      }
    
    if (check_eth_mac_txt(ETH_SRC))  // if '1' then user did not set MAC address (or problem occurred)
