@@ -20,21 +20,83 @@
 
 ////////////////////////////////////////////////////////////////////
 //
-// Layer 7 packet types
-// 
-// 1. DNS
+// DNS: Only UDP-based here 
 // 
 ////////////////////////////////////////////////////////////////////
 
 #include "mz.h"
+#include "cli.h"
+
+
+#define MZ_DNS_HELP \
+   		"| DNS type: Send Domain Name System Messages.\n" \
+		"|\n" \
+		"| Generally there are two interesting general DNS messages: queries and answers. The easiest\n" \
+		"| way is to use the following syntax:\n" \
+		"|\n" \
+		"|   query|q = <name>[:<type>]  ............. where type is per default \"A\"\n" \
+		"|                                            (and class is always \"IN\")\n" \
+		"|\n" \
+		"|   answer|a = [<type>:<ttl>:]<rdata> ...... ttl is per default 0.\n" \
+		"|            = [<type>:<ttl>:]<rdata>/[<type>:<ttl>:]<rdata>/...\n" \
+		"|\n" \
+		"| Note: If you only use the 'query' option then a query is sent. If you additonally add\n" \
+		"|       an 'answer' option then an answer is sent.\n" \
+		"|\n" \
+		"| Examples: \n" \
+		"|\n" \
+		"|   q = www.xyz.com\n" \
+		"|   q = www.xyz.com, a=192.168.1.10\n" \
+		"|   q = www.xyz.com, a=A:3600:192.168.1.10\n" \
+		"|   q = www.xyz.com, a=CNAME:3600:abc.com/A:3600:192.168.1.10\n" \
+		"|\n" \
+		"| Note: <type> can be: A, CNAME, or any integer\n" \
+		"|\n" \
+		"|\n" \
+		"| OPTIONAL parameter hacks: (if you don't know what you do this might cause invalid packets)\n" \
+		"|\n" \
+		"|   Parameter                    Description                               query / reply)\n" \
+		"|   -------------------------------------------------------------------------------------\n" \
+		"|\n" \
+		"|   request/response|reply ..... flag only                               request / n.a.  \n" \
+		"|   id ......................... packet id (0-65535)                     random  / random\n" \
+		"|   opcode (or op) ............. accepts values 0..15 or one of             std  / 0     \n" \
+	        "|                                these keywords: \n" \
+		"|         = std ................... Standard Query\n" \
+		"|         = inv ................... Inverse Query\n" \
+		"|         = sts ................... Server Status Request\n" \
+		"|   aa or !aa .................. Authoritative Answer                     UNSET  / SET\n" \
+		"|   tc or !tc .................. Truncation                               UNSET  / UNSET\n" \
+		"|   rd or !rd .................. Recursion Desired                          SET  / SET\n" \
+		"|   ra or !ra .................. Recursion Available                      UNSET  / SET\n" \
+		"|   z .......................... Reserved (takes values 0..7)                 0  / 0 \n" \
+		"|                                (z=2...authenticated)\n" \
+		"|   rcode ...................... Response Code (0..15); interesting           0  / 0 \n" \
+		"|                                values are:\n" \
+		"|        = 0 ...................... No Error Condition\n" \
+		"|        = 1 ...................... Unable to interprete query due to format error\n" \
+		"|        = 2 ...................... Unable to process due to server failure\n" \
+		"|        = 3 ...................... Name in query does not exist\n" \
+		"|        = 4 ...................... Type of query not supported\n" \
+		"|        = 5 ...................... Query refused\n" \
+		"|\n" \
+		"| Count values (values 0..65535) will be set automatically! You should not set these\n" \
+		"| values manually except you are interested in invalid packets.\n" \
+		"|   qdcount (or qdc) ........... Number of entries in question section        1  / 1\n" \
+		"|   ancount (or anc) ........... Number of RRs in answer records section      0  / 1\n" \
+		"|   nscount (or nsc) ........... Number of name server RRs in authority       0  / 0\n" \
+		"|                                records section\n" \
+		"|   arcount (or arc) ........... Number of RRs in additional records section  0  / 0\n" \
+		"\n"
+
 
 int dns_get_query (char* argval);
 int dns_get_answer (char* argval);
 
 
 
-// Note: we do NOT use libnet here (had difficultes with bugs there...)
-int  create_dns_packet ()
+// Note: I do NOT use libnet here (had problems with bugs there...)
+int create_dns_packet ()
 {
    
    char  *token, *tokenptr, argval[MAX_PAYLOAD_SIZE];
@@ -66,7 +128,7 @@ int  create_dns_packet ()
    // bit fields for dns_flags1: Q/R(1), OPCODE(4), AA(1), TC(1), RD(1)
    // bit fields for dns_flags0: RA(1), Z(3), RCODE(4)
    u_int8_t
-     dns_flags_qr,               // 1 bit
+     dns_flags_qr,                  // 1 bit
      dns_flags_opcode,              // 4 bits
      dns_flags_aa,                  // 1 bit
      dns_flags_tc,                  // 1 bit
@@ -87,74 +149,19 @@ int  create_dns_packet ()
    
    if ( (getarg(tx.arg_string,"help", NULL)==1) && (mode==DNS) )
      {
-	
-	fprintf(stderr,"\n"
-		MAUSEZAHN_VERSION
-		"\n"
-		"| DNS type: Send Domain Name System Messages.\n"
-		"|\n"
-		"| Generally there are two interesting general DNS messages: queries and answers. The easiest\n"
-		"| way is to use the following syntax:\n"
-		"|\n"
-		"|   query|q = <name>[:<type>]  ............. where type is per default \"A\"\n"
-		"|                                            (and class is always \"IN\")\n"
-		"|\n"
-		"|   answer|a = [<type>:<ttl>:]<rdata> ...... ttl is per default 0.\n"
-		"|            = [<type>:<ttl>:]<rdata>/[<type>:<ttl>:]<rdata>/...\n"
-		"|\n"
-		"| Note: If you only use the 'query' option then a query is sent. If you additonally add\n"
-		"|       an 'answer' then an answer is sent.\n"
-		"|\n"
-		"| Examples: \n"
-		"|\n"
-		"|   q = www.xyz.com\n"
-		"|   q = www.xyz.com, a=192.168.1.10\n"
-		"|   q = www.xyz.com, a=A:3600:192.168.1.10\n"
-		"|   q = www.xyz.com, a=CNAME:3600:abc.com/A:3600:192.168.1.10\n"
-		"|\n"
-		"| Note: <type> can be: A, CNAME, or any integer\n"
-		"|\n"
-		"|\n"
-		"| OPTIONAL parameter hacks: (if you don't know what you do this might cause invalid packets)\n"
-		"|\n"
-		"|   Parameter                    Description                               query / reply)\n"
-		"|   -------------------------------------------------------------------------------------\n"
-		"|\n"
-		"|   request/response|reply ..... flag only                               request / n.a.  \n"
-		"|   id ......................... packet id (0-65535)                     random  / random\n"
-		"|   opcode (or op) ............. accepts values 0..15 or one of             std  / 0     \n"
-	        "|                                these keywords: \n"
-		"|         = std ................... Standard Query\n"                    
-		"|         = inv ................... Inverse Query\n"
-		"|         = sts ................... Server Status Request\n"
-		"|   aa or !aa .................. Authoritative Answer                     UNSET  / SET\n"
-		"|   tc or !tc .................. Truncation                               UNSET  / UNSET\n"
-		"|   rd or !rd .................. Recursion Desired                          SET  / SET\n"
-		"|   ra or !ra .................. Recursion Available                      UNSET  / SET\n"
-		"|   z .......................... Reserved (takes values 0..7)                 0  / 0 \n"
-		"|                                (z=2...authenticated)\n"
-		"|   rcode ...................... Response Code (0..15); interesting           0  / 0 \n"
-		"|                                values are:\n"
-		"|        = 0 ...................... No Error Condition\n"
-		"|        = 1 ...................... Unable to interprete query due to format error\n"
-		"|        = 2 ...................... Unable to process due to server failure\n"
-		"|        = 3 ...................... Name in query does not exist\n"
-		"|        = 4 ...................... Type of query not supported\n"
-		"|        = 5 ...................... Query refused\n"
-		"|\n"
-		"| Count values (values 0..65535) will be set automatically! You should not set these\n"
-		"| values manually except you are interested in invalid packets.\n"
-		"|   qdcount (or qdc) ........... Number of entries in question section        1  / 1\n"
-		"|   ancount (or anc) ........... Number of RRs in answer records section      0  / 1\n"
-		"|   nscount (or nsc) ........... Number of name server RRs in authority       0  / 0\n"
-		"|                                records section\n"
-		"|   arcount (or arc) ........... Number of RRs in additional records section  0  / 0\n"
-		"|\n"
-		"\n");
-
-	exit(0);
+	if (mz_port)
+	  {
+	     cli_print(gcli, "%s", MZ_DNS_HELP);
+	     return -1;
+	  }
+	else
+	  {
+	     fprintf(stderr,"\n" 
+		     MAUSEZAHN_VERSION
+		     "\n%s", MZ_DNS_HELP);
+	     exit(0);
+	  }
      }
-
  
 
    // general defaults: 
@@ -164,7 +171,7 @@ int  create_dns_packet ()
    dns_id0           = 0x42;    // dns_id0 = (u_int8_t) ( ((float) rand()/RAND_MAX)*255);
    dns_id1           = 0x42; 
    
-   dns_flags_qr   = 0;        // request
+   dns_flags_qr      = 0;        // request
    dns_flags_opcode  = 0;        // 'standard query' (also for response!)
    
    dns_type0         = 1;        // A record
@@ -184,8 +191,7 @@ int  create_dns_packet ()
 	(getarg(tx.arg_string,"q", argval)==1) )
      {
 
-	(void) dns_get_query (argval); // returns the length in byte
-	dns_num_q0=1;
+	(void) dns_get_query (argval); // returns the length in byte	dns_num_q0=1;
 	
 	// copy the result from gbuf to our local buffer 'section':
 	for (j=0;j<gbuf_s;j++)
@@ -307,7 +313,7 @@ int  create_dns_packet ()
 	  }
      }
 
-   
+
    
    
    if (getarg(tx.arg_string,"aa",NULL)==1)
@@ -494,7 +500,6 @@ int  create_dns_packet ()
    
    //// Add sections to dns_packet:
 
-
    
    for (j=0; j<i; j++)
      {
@@ -633,7 +638,6 @@ int dns_process_label(char* label, u_int8_t *buf)
 	avoid_buffer_overflow++;	
 	for (j=0; j<cnt ;j++)
 	  {	     
-
 	     *buf = *token;
 	     buf++;
 	     avoid_buffer_overflow++;
@@ -784,7 +788,7 @@ int dns_get_answer(char* argval)
      }
    else              // Any other type 
      {
-	RDLEN = str2hex(field3, rdata); // should be 4 if IP address
+	RDLEN = str2hex(field3, rdata, 512); // should be 4 if IP address
      }
    
    ptrRDLEN = (u_int8_t*) &RDLEN;
