@@ -28,7 +28,7 @@
 // ****************************************************************************
 
 #include "mz.h"
-
+#include "cli.h"
 
 int send_eth()
 {
@@ -54,7 +54,8 @@ int send_eth()
      min_size=15,
      pad=0,
      repeat, loop, update,
-     i=0;
+     i=0,
+     j=0;
    char 
      err_buf[LIBNET_ERRBUF_SIZE],
      message[MAX_PAYLOAD_SIZE*3];
@@ -63,7 +64,8 @@ int send_eth()
    libnet_ptag_t  t;
    libnet_t       *l;
 
-
+   
+	
    if (tx.dot1Q)
      {
 	fprintf(stderr," Note: raw layer 2 mode does not support 802.1Q builder.\n"
@@ -78,30 +80,28 @@ int send_eth()
 	exit(1);
      }
    
+	
+
    
    // So other functions can use this function for sending Ethernet frames
    // These other functions must set dst, src, type and payload!
    if (tx.eth_params_already_set) goto ALL_SPECIFIED;
    
    
-   if ((tx.padding) && (tx.padding<15)) 
+   if ((tx.padding) && (tx.padding<15)) // Note: ignored if padding==0
      { 
-	tx.padding=15;
-	fprintf(stderr, " mz/send_eth: [Note] adjusted minimum frame size to 15 bytes.\n");
+	     tx.padding=15;
+	     if (mz_port)    {
+		     cli_print(gcli, "Note: Set padding to 15 bytes (total length)\n");
+	     } else
+		     fprintf(stderr, " mz/send_eth: [Note] adjusted minimum frame size to 15 bytes.\n");
      }
    
-   
-   
    // Create a temporal, local bytestring:
-   // 
-   bytestring_s = str2hex (tx.arg_string, bytestring);
-
-   // NEW: adjust to minimum frame length
-   if ( (bytestring_s<min_size) && (tx.padding<15) ) {
-     tx.padding=15;
-   }
-	
-	
+   //
+   for (i=0; i<MAX_PAYLOAD_SIZE; i++) bytestring[i]=0x00;
+   bytestring_s = str2hex (tx.arg_string, bytestring, MAX_PAYLOAD_SIZE);
+   
    // Set the flags to shorten subsequent decisions:
    src = strlen(tx.eth_src_txt);
    dst = strlen(tx.eth_dst_txt);
@@ -126,7 +126,7 @@ int send_eth()
 	// Evaluate Ethernet CLI options (-a and -b)
 	if (check_eth_mac_txt(ETH_DST))  // if true then problem!
 	  {
-	     str2hex("ff:ff:ff:ff:ff:ff",tx.eth_dst); // the default 
+	     str2hex("ff:ff:ff:ff:ff:ff",tx.eth_dst, 6); // the default 
 	  }
 	dst_random = tx.eth_dst_rand; // local vars are faster
      }
@@ -144,14 +144,13 @@ int send_eth()
 	// bytstring must contain src and type
 	min_size-=6;
      }
-   if ( (bytestring_s < min_size) &&  (tx.padding<15) )
+   if (bytestring_s < min_size)
      {
-	fprintf(stderr, " mz/send_eth: byte string too short (at least 15 bytes needed)!\n");
-	exit(1);
+	     j = min_size - bytestring_s; // this is what's missing
+	     bytestring_s += j; // note that bytestring has been filled up with 0x00, so we can do that
      }
 
-
-   
+	
    // ADDENDUM: If src specified, dst missing:
    // 
    if ( (!dst) && (src) ) 
@@ -166,28 +165,35 @@ int send_eth()
    if ((dst) && (!src))
      {
 	// Get eth_src from bytestring:
-	(void) getbytes (bytestring, tx.eth_src, byte_ptr, byte_ptr+5);
-	byte_ptr=7; // points to eth_type
+	if (bytestring_s>=6) {
+		(void) getbytes (bytestring, tx.eth_src, byte_ptr, byte_ptr+5);
+		byte_ptr=7; // now points to eth_type within bytestring
+	}
      }
    
    // FINALLY: If both dst and src have NOT been specified:
    // 
    if ((!dst) && (!src))
      {
-	(void) getbytes (bytestring, tx.eth_dst, byte_ptr, byte_ptr+5);
-	byte_ptr=7;
+	     if (bytestring_s>=6) {
+		     (void) getbytes (bytestring, tx.eth_dst, byte_ptr, byte_ptr+5);
+		     byte_ptr=7;
+	     }
 
-	(void) getbytes (bytestring, tx.eth_src, byte_ptr, byte_ptr+5);
-	byte_ptr=13; // points to eth_type
+	     if (bytestring_s>=12) {
+		     (void) getbytes (bytestring, tx.eth_src, byte_ptr, byte_ptr+5);
+		     byte_ptr=13; // points to eth_type
+	     }
      }
    
    // Set eth_type:
    // 
-   tx.eth_type = 256 * bytestring[byte_ptr-1] + bytestring[byte_ptr]; // don't forget: byte_ptr counts from 1 not 0
-   byte_ptr+=2; // points to first payload byte (if available)
+   if (bytestring_s>=2) {
+	   tx.eth_type = 256 * bytestring[byte_ptr-1] + bytestring[byte_ptr]; // don't forget: byte_ptr counts from 1 not 0
+	   byte_ptr+=2; // points to first payload byte (if available)
+   }
+
 
-   
-   
    // Get remaining payload:
    // 
    if ( (tx.eth_payload_s = bytestring_s - byte_ptr +1) > 0 ) // if there are any remaining bytes
@@ -196,12 +202,12 @@ int send_eth()
      }
    
 
+	
    // Add padding if desired. 
    // Note: padding means 'extend to given length' (not 'add these number of bytes')
    if (tx.padding) 
      {
 	pad = tx.padding - (14 + tx.eth_payload_s); // number of additonal pad bytes required
-	
 	for (i=0; i<pad; i++)
 	  {  
 	    // tx.eth_payload[i+tx.eth_payload_s] = (u_int8_t) ( ((float) rand()/RAND_MAX)*256);
@@ -211,6 +217,7 @@ int send_eth()
      }
    
    
+	
    ALL_SPECIFIED:
    // *** All Ethernet parameters have been determined !
    // *** Now let's send the frame!
@@ -303,13 +310,15 @@ int send_eth()
 		       fprintf(stderr, " %s\n",message);
 		    }
 		  update=0;
-		  if (verbose==2)
-		       {
-			       fprintf(stderr, "\n");
-			       fprintf(stderr, "*** NOTE: Simulation only! Nothing has been sent! ***\n");
-			       libnet_destroy(l);
-			       return 0;
-		       }
+		  if (verbose==2) 
+		    {
+		        fprintf(stderr, "\n");
+		       	fprintf(stderr, "*** NOTE: Simulation only! Nothing has been sent! ***\n");
+		       libnet_destroy(l);
+		       return 0;
+		    }
+		  
+		    
 	       }
 	     
 	     libnet_write(l);
