@@ -1,6 +1,6 @@
 /*
  * Mausezahn - A fast versatile traffic generator
- * Copyright (C) 2008 Herbert Haas
+ * Copyright (C) 2008-2010 Herbert Haas
  * 
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License version 2 as published by the 
@@ -25,6 +25,71 @@
 // ***************************************************************************
 
 #include "mz.h"
+#include "cli.h"
+
+
+
+#define MZ_ARP_HELP \
+   		"| ARP type: Send arbitrary ARP packets.\n" \
+		"| Note:\n" \
+		"| - The Ethernet dst and src MAC addresses can be specified but can be also 'rand'.\n" \
+		"| - If dst and src are NOT specified then practical defaults are used (src=own MAC, dst=bc).\n" \
+		"|\n" \
+		"|    ARGUMENT SYNTAX: <command> [<parameters>]\n" \
+		"|                         |           |\n" \
+		"|  help, request, reply --+           |\n" \
+		"|                                     +-- sendermac, senderip, targetmac, targetip\n" \
+		"|                                         smac       sip       tmac       tip\n" \
+		"|\n" \
+		"| EXAMPLES:\n" \
+		"|   1. Legitimate ARP response to broadcast:\n" \
+		"|      # mz eth0 -t arp \"reply\"\n" \
+		"|   2. ARP cache poisoning, claiming to be 192.168.0.1, telling a target PC:\n" \
+		"|      # mz eth0 -t arp \"reply, senderip=192.168.0.1, targetmac=00:00:0c:01:02:03, targetip=172.16.1.50\"\n" \
+		"\n"
+
+
+#define MZ_BPDU_HELP \
+   		"| BPDU type: Send arbitrary BPDU packets (spanning tree).\n" \
+		"|\n" \
+		"| ARGUMENT SYNTAX: <command> [<parameters>]\n" \
+		"|                      |           \n" \
+		"|          conf, tcn --+           \n" \
+		"|                                  \n" \
+		"| Parameters:\n" \
+		"|\n" \
+		"|      id = 0-65535      ..... default: 0, identifies 'Spanning Tree Protocol'\n" \
+		"| version = 0-255        ..... default: 0\n" \
+		"|    type = 0-255        ..... BPDU Type: 0=CONF, 1=TCN (default: CONF)\n" \
+		"|   flags = 0-255        ..... 1=TC, 128=ACK (default: 0 = No TC, No ACK)\n" \
+		"|  rootid = <pri>:<mac>  ..... 8 byte Root-ID (default: 00:00:<own-mac>)\n" \
+		"|  rootpc = 0-4294967295 ..... root path cost (default: 0)\n" \
+		"|     bid = <mac>        ..... 6 byte MAC address (default: own-mac)\n" \
+		"|     pid = 0-65535      ..... port identifier (default: 0)\n" \
+		"|     age = 0-65535      ..... message age (default: 0)\n" \
+		"|  maxage = 0-65535      ..... max age (default: 20)\n" \
+		"|   hello = 0-65535      ..... hello time (default: 2)\n" \
+		"|     fwd = 0-65535      ..... forward delay (default: 15)\n" \
+		"|     tag     -          ..... Keyword to enforce 802.1Q VLAN tag; use this\n" \
+		"|                              together with the 'vlan' parameter below.\n" \
+		"|\n" \
+		"| PVST+ extensions:\n" \
+		"|\n" \
+		"|  vlan     ..... VLAN number (default: 0)\n" \
+		"|  pri      ..... 802.1P-Priority (0-7, default: 0)\n" \
+		"|  notag    ..... Omit 802.1Q VLAN tag\n" \
+		"|  \n" \
+                "|\n" \
+		"| DEFAULTS: mz sends standard IEEE 802.1d (CST) BPDUs and assumes that your computer\n" \
+		"| wants to become the root bridge (rid=bid). Configuration BPDUs are the default but\n" \
+		"| can be changed using the 'tcn' keyword. Optionally the 802.3 source and destination\n" \
+		"| MAC addresses can be specified using the -a and -b options. Per default, the correct\n" \
+		"| STP or  PVST+ destination addresses are used (same as '-b stp' or '-b pvst', \n" \
+		"| respectively).\n" \
+		"| \n" \
+		"| Note that the parameter 'vlan' only selects the PVST+ mode if the parameter 'tag' is\n" \
+		"| NOT used.\n" \
+		"\n"
 
 
 
@@ -89,6 +154,21 @@ int send_arp ()
 	exit(1);
      }
 
+   if (getarg(tx.arg_string,"help", NULL)==1)
+     {
+	if (mz_port)
+	  {
+	     cli_print(gcli, "%s", MZ_ARP_HELP);
+	     return -1;
+	  }
+	else
+	  {
+	     fprintf(stderr,"\n" 
+		     MAUSEZAHN_VERSION
+		     "\n%s", MZ_ARP_HELP);
+	     exit(0);
+	  }
+     }
 
    
    // Set the flags to shorten subsequent decisions:
@@ -103,32 +183,6 @@ int send_arp ()
 	exit(EXIT_FAILURE);
      }
 
-  
-   if (getarg(tx.arg_string,"help", NULL)==1)
-     {
-	fprintf(stderr,"\n"
-		MAUSEZAHN_VERSION
-		"\n"
-		"| ARP type: Send arbitrary ARP packets.\n"
-		"| Note:\n"
-		"| - The Ethernet dst and src MAC addresses can be specified but can be also 'rand'.\n"
-		"| - If dst and src are NOT specified then practical defaults are used (src=own MAC, dst=bc).\n"
-		"|\n"	  
-		"|    ARGUMENT SYNTAX: <command> [<parameters>]\n"
-		"|                         |           |\n"
-		"|  help, request, reply --+           |\n"
-		"|                                     +-- sendermac, senderip, targetmac, targetip\n"
-		"|                                         smac       sip       tmac       tip\n"
-		"|\n"
-		"| EXAMPLES:\n"
-		"|   1. Legitimate ARP response to broadcast:\n"
-		"|      # mz eth0 -t arp \"reply\"\n"
-		"|   2. ARP cache poisoning, claiming to be 192.168.0.1, telling a target PC:\n"
-		"|      # mz eth0 -t arp \"reply, senderip=192.168.0.1, targetmac=00:00:0c:01:02:03, targetip=172.16.1.50\"\n"
-		"\n");
-	
-	exit(0);
-     }
    
    
    if (getarg(tx.arg_string,"request", NULL)==1)
@@ -153,7 +207,7 @@ int send_arp ()
    if ( (getarg(tx.arg_string,"sendermac", argval)==1) || (getarg(tx.arg_string,"smac", argval)==1) )
      {
 	//TODO: Allow 'rand' as sendermac
-	str2hex(argval,sendermac);
+	str2hex(argval,sendermac,6);
      }
    else
      {
@@ -164,16 +218,16 @@ int send_arp ()
 	
    if ( (getarg(tx.arg_string,"targetmac", argval)==1) || (getarg(tx.arg_string,"tmac", argval)==1) )
      {
-	str2hex(argval,targetmac);
+	str2hex(argval,targetmac,6);
 	tm=1;
      }
    else
      {  
 	// targetmac is either zero (request) or bcast (reply=>gratitious ARP)
 	if (arpmode==1) //request
-	  str2hex("00:00:00:00:00:00",targetmac);
+	  str2hex("00:00:00:00:00:00",targetmac, 6);
 	else //reply
-	  str2hex("ff:ff:ff:ff:ff:ff",targetmac);
+	  str2hex("ff:ff:ff:ff:ff:ff",targetmac, 6);
      }
 
    
@@ -231,7 +285,7 @@ int send_arp ()
    if ((!dst) && (!src)) // ... user does not care about addresses (both eth_dst and eth_src NOT specified)
      {
 	if (arpmode==1)
-	  str2hex("ff:ff:ff:ff:ff:ff", tx.eth_dst);
+	  str2hex("ff:ff:ff:ff:ff:ff", tx.eth_dst, 6);
 	else
 	  getbytes(targetmac, tx.eth_dst, 1, 6); // either also bcast or specific MAC
 	
@@ -251,7 +305,7 @@ int send_arp ()
 	if (!dst)
 	  {
 	     if (arpmode==1)
-	       str2hex("ff:ff:ff:ff:ff:ff", tx.eth_dst);
+	       str2hex("ff:ff:ff:ff:ff:ff", tx.eth_dst, 6);
 	     else
 	       getbytes(targetmac, tx.eth_dst, 1, 6); // either also bcast when reply or specific MAC
 	  }
@@ -259,7 +313,7 @@ int send_arp ()
 	  {
 	     if (check_eth_mac_txt(ETH_DST))  // if true then problem!
 	       {
-		  str2hex("ff:ff:ff:ff:ff:ff",tx.eth_dst); // the default 
+		  str2hex("ff:ff:ff:ff:ff:ff",tx.eth_dst, 6); // the default
 	       }
 	  }
 	
@@ -272,7 +326,7 @@ int send_arp ()
 	  {
 	     if (check_eth_mac_txt(ETH_SRC))  // if true then problem!
 	       {
-		  str2hex("ff:ff:ff:ff:ff:ff",tx.eth_src); // the default 
+		  str2hex("ff:ff:ff:ff:ff:ff",tx.eth_src, 6); // the default
 	       }
 	  }
 	
@@ -302,7 +356,8 @@ int send_arp ()
    
    for (i=1; i<=loop; i++)
      {
-	libnet_write(l);
+	     
+	if (!simulate) libnet_write(l);
 	
 	if (verbose)
 	  {
@@ -326,7 +381,7 @@ int send_arp ()
    
    libnet_destroy(l);
    
-   return 1;
+   return 0;
 } 
 
 
@@ -344,7 +399,6 @@ int send_arp ()
 
 
 ///////////////////////////////////////////////////////////////////////////////////////
-
 // Send arbitrary BPDU frames.
 //
 // commands: 
@@ -423,12 +477,29 @@ int send_bpdu ()
 	exit(1);
      }
 
-   
+
+   // HELP TEXT
+   if (getarg(tx.arg_string,"help", NULL)==1)
+     {
+	if (mz_port)
+	  {
+	     cli_print(gcli, "%s", MZ_BPDU_HELP);
+	     return -1;
+	  }
+	else
+	  {
+	     fprintf(stderr,"\n" 
+		     MAUSEZAHN_VERSION
+		     "\n%s", MZ_BPDU_HELP);
+	     exit(0);
+	  }
+     }
+
    /////////////////////////////////////////////////////////
    // Default Destination Address
    if (check_eth_mac_txt(ETH_DST))  // if true then problem!
      {
-	str2hex("01:80:C2:00:00:00",tx.eth_dst); // if '1' then user did not set MAC address (or problem occurred)
+	str2hex("01:80:C2:00:00:00",tx.eth_dst, 6); // if '1' then user did not set MAC address (or problem occurred)
      }
    
    // Default Bridge-ID
@@ -439,54 +510,6 @@ int send_bpdu ()
    /////////////////////////////////////////////////////////
    
    
-   // HELP TEXT
-   if (getarg(tx.arg_string,"help", NULL)==1)
-     {
-	fprintf(stderr,"\n"
-		MAUSEZAHN_VERSION
-		"\n"
-		"| BPDU type: Send arbitrary BPDU packets (spanning tree).\n"
-		"|\n"
-		"| ARGUMENT SYNTAX: <command> [<parameters>]\n"
-		"|                      |           \n"
-		"|          conf, tcn --+           \n"
-		"|                                  \n"
-		"| Parameters:\n"
-		"|\n"
-		"|      id = 0-65535      ..... default: 0, identifies 'Spanning Tree Protocol'\n"
-		"| version = 0-255        ..... default: 0\n"
-		"|    type = 0-255        ..... BPDU Type: 0=CONF, 1=TCN (default: CONF)\n" 
-		"|   flags = 0-255        ..... 1=TC, 128=ACK (default: 0 = No TC, No ACK)\n"
-		"|  rootid = <pri>:<mac>  ..... 8 byte Root-ID (default: 00:00:<own-mac>)\n"
-		"|  rootpc = 0-4294967295 ..... root path cost (default: 0)\n"
-		"|     bid = <mac>        ..... 6 byte MAC address (default: own-mac)\n"
-		"|     pid = 0-65535      ..... port identifier (default: 0)\n"
-		"|     age = 0-65535      ..... message age (default: 0)\n"
-		"|  maxage = 0-65535      ..... max age (default: 20)\n"
-		"|   hello = 0-65535      ..... hello time (default: 2)\n"
-		"|     fwd = 0-65535      ..... forward delay (default: 15)\n"
-		"|     tag     -          ..... Keyword to enforce 802.1Q VLAN tag; use this\n"
-		"|                              together with the 'vlan' parameter below.\n"
-		"|\n"
-		"| PVST+ extensions:\n"
-		"|\n"
-		"|  vlan     ..... VLAN number (default: 0)\n"
-		"|  pri      ..... Priority (0-7, default: 0)\n"
-		"|  notag    ..... Omit 802.1Q VLAN tag\n"
-		"|  \n"
-                "|\n"
-		"| DEFAULTS: mz sends standard IEEE 802.1d (CST) BPDUs and assumes that your computer\n"
-		"| wants to become the root bridge (rid=bid). Configuration BPDUs are the default but\n"
-		"| can be changed using the 'tcn' keyword. Optionally the 802.3 source and destination\n"
-		"| MAC addresses can be specified using the -a and -b options. Per default, the correct\n"
-		"| STP or  PVST+ destination addresses are used (same as '-b stp' or '-b pvst', \n"
-		"| respectively).\n"
-		"| \n"
-		"| Note that the parameter 'vlan' only selects the PVST+ mode if the parameter 'tag' is\n"
-		"| NOT used.\n"
-		"\n");
-	exit(0);
-     }
    
    
    // determine BPDU type:
@@ -534,7 +557,7 @@ int send_bpdu ()
    
    if (getarg(tx.arg_string,"rid", argval)==1)
      {
-	if (str2hex(argval,root_id)!=8)
+	if (str2hex(argval,root_id, 8)!=8)
 	  {
 	     fprintf(stderr," mz/send_bpdu: [ERROR] The root-id must be exactly 8 bytes!\n");
 	     exit (-1);
@@ -548,7 +571,7 @@ int send_bpdu ()
 
    if (getarg(tx.arg_string,"bid", argval)==1)
      {
-	if (str2hex(argval,bridge_id)!=6)
+	if (str2hex(argval,bridge_id, 6)!=6)
 	  {
 	     fprintf(stderr," mz/send_bpdu: [ERROR] The bridge-id must be exactly 6 bytes!\n");
 	     exit (-1);
@@ -589,10 +612,12 @@ int send_bpdu ()
 	// 
 	if (check_eth_mac_txt(ETH_DST))  // if '1' then user did not set MAC address (or problem occurred)
 	  {
-	     str2hex("01:00:0C:CC:CC:CD",tx.eth_dst); // Cisco PVST+ address
+	     str2hex("01:00:0C:CC:CC:CD",tx.eth_dst, 6); // Cisco PVST+ address
 	  }
 	
-	bpdu_payload[0] = 0x34;
+/*	// OLD TLV, maybe wrong, maybe obsolete, I don't know.
+        
+        bpdu_payload[0] = 0x34;
 	bpdu_payload[1] = 0x00;
 	bpdu_payload[2] = 0x02;
 	vlan = (u_int16_t) str2int(argval);
@@ -602,9 +627,20 @@ int send_bpdu ()
 	bpdu_payload[4] = *(x);
 	bpdu_payload[5] = 0x00;
 	bpdu_payload[6] = 0x00;
-	//bpdu_payload[7] = 0x00;
 	bpdu_payload_s = 7;
-	
+*/
+        // Updated PVST+ TLV:
+     	bpdu_payload[0] = 0x00;
+	bpdu_payload[1] = 0x00;
+	bpdu_payload[2] = 0x00;
+	bpdu_payload[3] = 0x00;
+	bpdu_payload[4] = 0x02;
+        vlan = (u_int16_t) str2int(argval);
+        x = (u_int8_t*) &vlan;
+	bpdu_payload[5] = *(x+1);
+	bpdu_payload[6] = *(x);
+	bpdu_payload_s = 7;
+	     
 	tag=1; // set the default: Use 802.1Q tag !!!
      }
    else // even a normal BPDU must be padded to 60 bytes (total)
@@ -649,7 +685,7 @@ int send_bpdu ()
 	// 
 	if (check_eth_mac_txt(ETH_DST))  // if '1' then user did not set MAC address (or problem occurred)
 	  {
-	     str2hex("01:80:C2:00:00:00",tx.eth_dst); 
+	     str2hex("01:80:C2:00:00:00",tx.eth_dst, 6);
 	  }
 	vlan = (u_int16_t) str2int(argval);
      }
@@ -824,7 +860,7 @@ int send_bpdu ()
    
    for (i=1; i<=loop; i++)
      {
-	libnet_write(l);
+	if (!simulate) libnet_write(l);
 	
 	if (verbose)
 	  {
@@ -861,6 +897,6 @@ int send_bpdu ()
    
    libnet_destroy(l); 
 
-   return 1;
+   return 0;
 }
 
