@@ -1,6 +1,6 @@
 /*
  * Mausezahn - A fast versatile traffic generator
- * Copyright (C) 2008 Herbert Haas
+ * Copyright (C) 2008-2010 Herbert Haas
  * 
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License version 2 as published by the 
@@ -29,10 +29,104 @@
 ////////////////////////////////////////////////////////////////////
 
 #include "mz.h"
+#include "cli.h"
+
+
+#define MZ_UDP_HELP \
+   		"| UDP type: Send raw UDP packets.\n"  \
+		"|\n" \
+		"| Parameters: \n" \
+		"|\n" \
+		"|  sp          0-65535\n" \
+		"|  dp          0-65535\n" \
+		"|  len         0-65535\n" \
+		"|  sum         0-65535\n" \
+		"|  payload|p   <hex payload>\n" \
+		"|\n" \
+		"| Optionally the port numbers can be specified as ranges, e. g. \"dp=1023-33700\",\n" \
+		"| in which case one packet per port number is sent.\n" \
+		"|\n" \
+		"| Note that the UDP length must include the header length. If you do NOT specify the len\n" \
+		"| parameter (or specify len=0) then Mausezahn will compute the correct length.\n" \
+		"|\n" \
+		"| Note that all IP parameters can be modified (see IP help, i. e. '-t ip \"help\")\n" \
+		"| except that (to avoid confusion) the IP length is 'iplen' and the IP checksum is 'ipsum'.\n" \
+		"| Of course all Ethernet fields can also be accessed.\n" \
+		"|\n" \
+		"\n"
+
+
+#define MZ_ICMP_HELP \
+		"| ICMP type: Send raw ICMP packets.\n" \
+		"|\n" \
+		"| ARGUMENT SYNTAX: [type] <optional parameters> \n" \
+		"| \n" \
+		"| Per default an echo reply is sent (type=0, code=0)\n" \
+		"|\n" \
+		"| TYPE         OPTIONAL PARAMETERS\n" \
+		"| ===========  ====================================================================\n" \
+		"| Ping:       \"ping\" or \"echoreq\" \n" \
+		"|              'id' (0-65535) is the optional identification number\n" \
+		"|              'seq' (0-65535) is the optional packet sequence number\n" \
+		"|\n" \
+		"| Redirect:   \"redir, code=0, gw=192.168.1.10, p=aa:bb:cc\"\n" \
+		"|              'gw' (or 'gateway') is the announced gateway, by default your own\n" \
+		"|              IP address.\n" \
+		"|              'code' can be:\n" \
+		"|                  0 ... redirect datagram for the network\n" \
+		"|                  1 ... redirect datagram for the host\n" \
+		"|                  2 ... redirect datagram for ToS and network\n" \
+		"|                  3 ... redirect datagram for ToS and host\n" \
+		"|              'p' (or 'payload') is the payload of the ICMP packet, tpyically an IP\n" \
+		"|              header. Note that - at the moment - you must prepare this payload by\n"  \
+		"|              yourself.\n" \
+		"|\n" \
+		"| Unreachable \"unreach, code=2\"\n" \
+		"|              'code' can be:\n" \
+		"|                  0 ... network unreachable\n" \
+		"|                  1 ... host unreachable\n" \
+		"|                  2 ... protocol unreachable\n" \
+		"|                  3 ... port unreachable\n" \
+		"|                  4 ... fragmentation needed but DF-bit is set\n" \
+		"|                  5 ... source route failed\n" \
+		"|\n" \
+		"|\n" \
+		"| (other ICMP types will follow)\n" \
+		"|\n" \
+		"\n"
+
+#define MZ_TCP_HELP \
+   		"| TCP type: Send raw TCP packets.\n" \
+		"|\n" \
+		"| Parameters  Values                               Explanation \n"  \
+		"| ----------  ------------------------------------ -------------------\n" \
+		"|  sp         0-65535                              Source Port\n" \
+		"|  dp         0-65535                              Destination Port\n" \
+		"|  flags      fin|syn|rst|psh|ack|urg|ecn|cwr\n" \
+		"|  s          0-4294967295                         Sequence Nr.\n" \
+		"|  a          0-4294967295                         Acknowledgement Nr.\n" \
+		"|  win        0-65535                              Window Size\n" \
+		"|  urg        0-65535                              Urgent Pointer\n" \
+		"|  sum        0-65535                              Checksum\n" \
+		"|\n" \
+		"| The port numbers can be specified as ranges, e. g. \"dp=1023-33700\".\n" \
+		"| Multiple flags can be specified such as \"flags=syn|ack|urg\".\n" \
+		"|\n" \
+		"| Also the sequence number can be specified as a range, for example:\n" \
+		"|\n" \
+		"|   s=10000-50000 ... send 40000 packets with SQNRs in that range. If the second\n" \
+		"|                     value is lower than the first then it is assumed that the\n" \
+		"|                     SQNRs should 'wrap around'.\n" \
+		"|   ds=30000 ........ use this increment within a SQNR-range.\n" \
+		"|\n" \
+		"| Note that all IP parameters can be modified (see IP help, i. e. '-t ip \"help\")\n" \
+		"| except that (to avoid confusion) the IP length is 'iplen' and the IP checksum is 'ipsum'.\n" \
+		"| Of course all Ethernet fields can also be accessed.\n"\
+		"|\n"
 
 
 
-// Note: If another function specifed tx.udp_payload then it must also
+// Note: If another function specified tx.udp_payload then it must also
 // set tx.udp_payload_s AND tx.udp_len = tx.udp_payload_s + 8
 libnet_ptag_t  create_udp_packet (libnet_t *l)
 {
@@ -50,35 +144,24 @@ libnet_ptag_t  create_udp_packet (libnet_t *l)
    
    if ( (getarg(tx.arg_string,"help", NULL)==1) && (mode==UDP) )
      {
-	
-	fprintf(stderr,"\n"
-		MAUSEZAHN_VERSION
-		"\n"
-		"| UDP type: Send raw UDP packets.\n"
-		"|\n"
-		"| Parameters: \n"
-		"|\n"
-		"|  sp     0-65535\n"
-		"|  dp     0-65535\n"
-		"|  len    0-65535\n"
-		"|  sum    0-65535\n"
-		"|\n"
-		"| Optionally the port numbers can be specified as ranges, e. g. \"dp=1023-33700\",\n"
-		"| in which case one packet per port number is sent.\n"
-		"|\n"
-		"| Note that the UDP length must include the header length. If you do NOT specify the len\n"
-		"| parameter (or specify len=0) then Mausezahn will compute the correct length.\n"
-		"|\n"
-		"| Note that all IP parameters can be modified (see IP help, i. e. '-t ip \"help\")\n"
-		"| except that (to avoid confusion) the IP length is 'iplen' and the IP checksum is 'ipsum'.\n"
-		"| Of course all Ethernet fields can also be accessed.\n"
-		"|\n"
-		"\n");
+	if (mz_port)
+	  {
+	     cli_print(gcli, "%s", MZ_UDP_HELP);
+	     return -1;
+	  }
+	else
+	  {
+	     
+	     fprintf(stderr,"\n" 
+		     MAUSEZAHN_VERSION
+		     "\n%s", MZ_UDP_HELP);
 
-	exit(0);
+	     exit(0);
+	  }
+	
      }
 
- 
+   
    // Evaluate CLI parameters:
 
    if (getarg(tx.arg_string,"dp", argval)==1)
@@ -97,10 +180,17 @@ libnet_ptag_t  create_udp_packet (libnet_t *l)
 	  }
      }
    
+
+   // Check if hex_payload already specified (externally)
+   if (tx.hex_payload_s)
+     {
+	memcpy( (void*) tx.udp_payload, (void*) tx.hex_payload, tx.hex_payload_s);
+	tx.udp_payload_s = tx.hex_payload_s;
+     }
    
    if ( (getarg(tx.arg_string,"payload", argval)==1) || (getarg(tx.arg_string,"p", argval)==1))
      {
-	tx.udp_payload_s = str2hex (argval, tx.udp_payload);
+	tx.udp_payload_s = str2hex (argval, tx.udp_payload, MAX_PAYLOAD_SIZE);
      }
    
    
@@ -116,6 +206,7 @@ libnet_ptag_t  create_udp_packet (libnet_t *l)
      {
 	strncpy((char *)tx.udp_payload, (char *)tx.ascii_payload, MAX_PAYLOAD_SIZE);
 	tx.udp_payload_s = strlen((char *)tx.ascii_payload);
+	printf("[%s]\n", tx.ascii_payload);
      }
 
 
@@ -132,7 +223,6 @@ libnet_ptag_t  create_udp_packet (libnet_t *l)
 	     tx.udp_payload[tx.udp_payload_s+i] = 0x42; // pad with THE ANSWER (why random?)
 	  }
 	tx.udp_payload_s += tx.padding;
-	
      }
 
    
@@ -177,6 +267,7 @@ libnet_ptag_t  create_udp_packet (libnet_t *l)
 	exit (0);
      }
 
+
    
    return t;
    
@@ -203,7 +294,6 @@ libnet_ptag_t  create_icmp_packet (libnet_t *l)
    libnet_ptag_t  t;
    char argval[MAX_PAYLOAD_SIZE];
    unsigned char *x;
-   
 
    int i;
    
@@ -214,55 +304,23 @@ libnet_ptag_t  create_icmp_packet (libnet_t *l)
 	UNREACHABLE
      } 
    icmp;           // which ICMP Type? 
-   
 
    
    if ( (getarg(tx.arg_string,"help", NULL)==1) && (mode==ICMP) )
      {
-	
-	fprintf(stderr,"\n"
-		MAUSEZAHN_VERSION
-		"\n"
-		"| ICMP type: Send raw ICMP packets.\n"
-		"|\n"
-		"| ARGUMENT SYNTAX: [type] <optional parameters> \n"
-		"| \n"
-		"| Per default an echo reply is sent (type=0, code=0)\n"
-		"|\n"
-		"| TYPE         OPTIONAL PARAMETERS\n"
-		"| ===========  ====================================================================\n"
-		"| Ping:       \"ping\" or \"echoreq\" \n"
-		"|              'id' (0-65535) is the optional identification number\n"
-		"|              'seq' (0-65535) is the optional packet sequence number\n"
-		"|\n"
-		"| Redirect:   \"redir, code=0, gw=192.168.1.10, p=aa:bb:cc\"\n"
-		"|              'gw' (or 'gateway') is the announced gateway, by default your own\n"
-		"|              IP address.\n"
-		"|              'code' can be:\n"
-		"|                  0 ... redirect datagram for the network\n"
-		"|                  1 ... redirect datagram for the host\n"
-		"|                  2 ... redirect datagram for ToS and network\n"
-		"|                  3 ... redirect datagram for ToS and host\n"
-		"|              'p' (or 'payload') is the payload of the ICMP packet, tpyically an IP\n"
-		"|              header. Note that - at the moment - you must prepare this payload by\n"
-		"|              yourself.\n"
-		"|\n"
-		"| Unreachable \"unreach, code=2\"\n"
-		"|              'code' can be:\n"
-		"|                  0 ... network unreachable\n"
-		"|                  1 ... host unreachable\n"
-		"|                  2 ... protocol unreachable\n"
-		"|                  3 ... port unreachable\n"
-		"|                  4 ... fragmentation needed but DF-bit is set\n"
-		"|                  5 ... source route failed\n"
-		"|\n"
-		"|\n"
-		"| (other ICMP types will follow)\n"
-		"|\n"
-		"|\n"
-		"\n");
-
-	exit(0);
+	if (mz_port)  
+	  {
+	     cli_print(gcli, "%s", MZ_ICMP_HELP);
+	     return -1;
+	  }
+	else 
+	  {
+	     
+	     fprintf(stderr,"\n" 
+		     MAUSEZAHN_VERSION
+		     "\n%s", MZ_ICMP_HELP);
+	     exit(0);
+	  }
      }
 
  
@@ -343,15 +401,24 @@ libnet_ptag_t  create_icmp_packet (libnet_t *l)
      }
    
    
+   // Check if hex_payload already specified (externally)
+   if (tx.hex_payload_s)
+     {
+	memcpy( (void*) tx.icmp_payload, (void*) tx.hex_payload, tx.hex_payload_s);
+	tx.icmp_payload_s = tx.hex_payload_s;
+     }
+   
+   
    if ( (getarg(tx.arg_string,"payload", argval)==1) || (getarg(tx.arg_string,"p", argval)==1))
      {
-	tx.icmp_payload_s = str2hex (argval, tx.icmp_payload);
+	tx.icmp_payload_s = str2hex (argval, tx.icmp_payload, MAX_PAYLOAD_SIZE);
      }
    else
      {
 	tx.icmp_payload_s = 0;
      }
-
+   
+   
    if (tx.ascii) // ASCII PAYLOAD overrides hex payload
      {
 	strncpy((char *)tx.icmp_payload, (char *)tx.ascii_payload, MAX_PAYLOAD_SIZE);
@@ -462,7 +529,7 @@ libnet_ptag_t  create_icmp_packet (libnet_t *l)
 ///////////////////////////////////////////////////
 
 
-// Note: If another function specifed tx.tcp_payload then it must also
+// Note: If another function specified tx.tcp_payload then it must also
 // set tx.tcp_payload_s AND tx.tcp_len = tx.tcp_payload_s + 20
 libnet_ptag_t  create_tcp_packet (libnet_t *l)
 {
@@ -491,40 +558,18 @@ libnet_ptag_t  create_tcp_packet (libnet_t *l)
    
    if ( (getarg(tx.arg_string,"help", NULL)==1) && (mode==TCP) )
      {
-	
-	fprintf(stderr,"\n"
-		MAUSEZAHN_VERSION
-		"\n"
-		"| TCP type: Send raw TCP packets.\n"
-		"|\n"
-		"| Parameters  Values                               Explanation \n"
-		"| ----------  ------------------------------------ -------------------\n"
-		"|  sp         0-65535                              Source Port\n"
-		"|  dp         0-65535                              Destination Port\n"
-		"|  flags      fin|syn|rst|psh|ack|urg|ecn|cwr\n"
-		"|  s          0-4294967295                         Sequence Nr.\n"
-		"|  a          0-4294967295                         Acknowledgement Nr.\n"
-		"|  win        0-65535                              Window Size\n"
-		"|  urg        0-65535                              Urgent Pointer\n"
-		"|  sum        0-65535                              Checksum\n"
-		"|\n"
-		"| The port numbers can be specified as ranges, e. g. \"dp=1023-33700\".\n"
-		"| Multiple flags can be specified such as \"flags=syn|ack|urg\".\n"
-		"|\n"		
-		"| Also the sequence number can be specified as a range, for example:\n"
-		"|\n"
-		"|   s=10000-50000 ... send 40000 packets with SQNRs in that range. If the second\n"
-		"|                     value is lower than the first then it is assumed that the\n"
-		"|                     SQNRs should 'wrap around'.\n"
-		"|   ds=30000 ........ use this increment within a SQNR-range.\n"
-		"|\n"
-		"| Note that all IP parameters can be modified (see IP help, i. e. '-t ip \"help\")\n"
-		"| except that (to avoid confusion) the IP length is 'iplen' and the IP checksum is 'ipsum'.\n"
-		"| Of course all Ethernet fields can also be accessed.\n"
-		"|\n"
-		"\n");
-
-	exit(0);
+	if (mz_port)
+	  {
+	     cli_print(gcli, "%s", MZ_TCP_HELP);
+	     return -1;
+	  }
+	else
+	  {
+	     fprintf(stderr,"\n" 
+		     MAUSEZAHN_VERSION
+		     "\n%s", MZ_TCP_HELP);
+	     exit(0);
+	  }
      }
 
  
@@ -595,10 +640,18 @@ libnet_ptag_t  create_tcp_packet (libnet_t *l)
 	  }
      }
    
+
+   // Check if hex_payload already specified (externally)
+   if (tx.hex_payload_s)
+     {
+	memcpy( (void*) tx.tcp_payload, (void*) tx.hex_payload, tx.hex_payload_s);
+	tx.tcp_payload_s = tx.hex_payload_s;
+     }
+   
    
    if ( (getarg(tx.arg_string,"payload", argval)==1) || (getarg(tx.arg_string,"p", argval)==1))
      {
-	tx.tcp_payload_s = str2hex (argval, tx.tcp_payload);
+	tx.tcp_payload_s = str2hex (argval, tx.tcp_payload, MAX_PAYLOAD_SIZE);
      }
    
 
@@ -625,17 +678,15 @@ libnet_ptag_t  create_tcp_packet (libnet_t *l)
 	     tx.tcp_payload[tx.tcp_payload_s+i] = 0x42; // pad with THE ANSWER (why random?)
 	  }
 	tx.tcp_payload_s += tx.padding;
-	tx.tcp_len = 20 + tx.tcp_payload_s;   // only needed by libnet to calculate checksum
-	tx.ip_payload_s = tx.tcp_len;         // for create_ip_packet
+
      }
 
 
    
-   tx.tcp_len = 20 + tx.tcp_payload_s;
-   tx.ip_payload_s = 20 + tx.tcp_payload_s;  // for create_ip_packet
+   tx.tcp_len = 20 + tx.tcp_payload_s;       // only needed by libnet to calculate checksum
+   tx.ip_payload_s = tx.tcp_len;         // for create_ip_packet
 
-
-   if (tx.tcp_control&0x02) // packets with syn require an option and have no payload!
+   if (tx.tcp_control & 0x02) // packets with syn require an MSS option 
      {
 	t2 = libnet_build_tcp_options(tcp_default_options,
 				     20, 
