@@ -1,6 +1,6 @@
 /*
  * Mausezahn - A fast versatile traffic generator
- * Copyright (C) 2009 Herbert Haas
+ * Copyright (C) 2008-2010 Herbert Haas
  * 
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License version 2 as published by the 
@@ -21,6 +21,8 @@
 #ifndef __MAUSEZAHN__
 #define __MAUSEZAHN__
 
+#define _GNU_SOURCE
+
 #include <libnet.h>
 #include <pcap.h>
 #include <stdio.h>
@@ -35,6 +37,11 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <arpa/inet.h>
+#include <sys/ioctl.h>
+#include <netinet/in.h>
+#include <net/if.h>
+#include <stdarg.h>
+#include <math.h>
 
 //#include <ctype.h>
 
@@ -42,15 +49,22 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////
 //
 //
-#define MAUSEZAHN_VERSION "Mausezahn 0.34.9 - (C) 2007-2009 by Herbert Haas - http://www.perihel.at/sec/mz/"
-#define MAUSEZAHN_VERSION_SHORT "0.34.9"
+#define MAUSEZAHN_VERSION "Mausezahn 0.40 - (C) 2007-2010 by Herbert Haas - http://www.perihel.at/sec/mz/"
+#define MAUSEZAHN_VERSION_SHORT "0.40"
 //
 //
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////
 
 
-#define SLEEP usleep               // The sleep function to use. 
+// "Dies ist ein schrecklicher Ort."
+
+#define MZ_DEFAULT_CONFIG_PATH "/etc/mausezahn/"   // see also mz_default_config_path below
+#define MZ_DEFAULT_LOG_PATH "/var/log/mausezahn/"  // see also mz_default_log_path below
+
+#define SLEEP usleep               // The sleep function to use. Consider 'nanosleep' in future.
 #define DEFAULT_DELAY 0
+#define PCAP_READ_TIMEOUT_MSEC 1   // The read timeout for pcap_open_live() 
+#define MZ_MAX_DEVICES 10          // Max number of network devices supported
 #define MAX_PAYLOAD_SIZE 3*8192
 #define MAX_DNS_NAME 256
 #define MAX_8021Q_TAGS 16         
@@ -58,27 +72,79 @@
 #define TIME_COUNT 100             // the default used-size of the timestamp arrays timeRX and timeTX
 #define MAX_DATA_BLOCKS 1000       // how many data blocks of size TIME_COUNT-1 should be written per file
 #define MAXBYTES_TO_READ 1500      // how many bytes the pcap routine should read from net
+#define RCV_RTP_MAX_BAR_WIDTH 500  // max line-width printed in BAR mode (see rcv_rtp.c)
 
-#define ETH_SRC 1   // These are only some symbols used by some functions.
+#define ETH_SRC 1   // These are only some symbols used by some functions. (Don't touch)
 #define ETH_DST 2   // These are only some symbols used by some functions.
 #define SRC_PORT 1  // These are only some symbols used by some functions.
 #define DST_PORT 2  // These are only some symbols used by some functions.
 
-#define TEST printf("HERE\n");
+#define TEST fprintf(stderr, "HERE at line %i in file %s\n", __LINE__,__FILE__ ); fflush(stderr);
 
 
 // -----  PCAP-specific definitions: ---------------------
 #define IPADDRSIZE 46
 
+
+int MZ_SIZE_LONG_INT;
+
+char mz_default_config_path[256];
+char mz_default_log_path[256];
+
+
+struct arp_table_struct {
+	int               index;     // an entry index (1, 2, ...) for easier user access
+	u_int8_t          sa[6];     // sent by this MAC SA
+	u_int8_t          smac[6];   // announced MAC
+	u_int8_t          smac_prev[6];   // previously announced MAC
+	u_int8_t          sip[4];    // announced IP
+	unsigned long int uni_rq;    // count unidirectional ARP requests for this IP
+	unsigned long int bc_resp;   // count broadcast ARP responses for this IP
+	unsigned long int uni_resp;  // count normal (unidir) ARP responses for this IP
+	unsigned long int changed;   // count how often the MAC address has changed!
+	int               locked;    // 1=this entry cannot be overidden anymore
+	int               dynamic;   // 1=learned dynamically, 0=configured by user
+	int               flags;     // anomaly information (length anomaly: bit 0, sa!=smac: bit 1 , ...)
+	int               gw;        // 1=Default GW
+	char              when[10];  // human readable timestamp (e. g. "11:42:53")
+	u_int32_t sec, nsec;         // timestamp of last ARP response
+	u_int32_t sec_prev, nsec_prev;         // timestamp of previous ARP response
+	//-----------------//
+	struct arp_table_struct *next;
+};
+
 // Device list
-struct
+struct device_struct
 {
-   char dev[16];
-   char ip[IPADDRSIZE+1];
-} device_list[10];
+	char       dev[16];               // Device name
+	int        index;                 // Device index (assigned by OS)
+	int        phy;                   // 1 if physical, 0 if not (e. g. loopback)
+	int        mtu;
+	int        cli;                   // if set to 1 then the CLI connection must terminate here
+	int        mgmt_only;             // if set to 1 then no data traffic is allowed through that interface
+	// ---- MAC addresses ----
+	u_int8_t   mac[6];                // Real MAC address
+	u_int8_t   mac_mops[6];           // MAC address to be used
+	// ---- IP related -----
+	char       ip_str[IPADDRSIZE+1];  // Real IP address as string in dotted decimal notation		
+	u_int8_t   ip[4];                 // Real IP address
+	u_int8_t   net[4];                // Real network
+	u_int8_t   mask[4];               // Real mask
+	u_int8_t   ip_mops[4];            // IP address to be used
+	// ---- Default Gateway per interface:
+	u_int8_t   mac_gw[6];             // MAC address of default gateway
+	u_int8_t   ip_gw[4];              // IP address of default gateway
+	// ---- various device-specific handles ----
+	pthread_t  arprx_thread;            
+   	pcap_t     *p_arp;                  // pcap handle
+	struct arp_table_struct *arp_table; // dedicated ARP table
+	int        ps;                    // packet socket
+} device_list[MZ_MAX_DEVICES];
+
 int device_list_entries;
                
-// Ethernet header 
+
+#pragma pack(1)
 struct struct_ethernet
 {
       u_int8_t   eth_da[6];
@@ -86,6 +152,22 @@ struct struct_ethernet
       u_int16_t  eth_type;
 };
 
+struct struct_arp
+{
+   u_int16_t arp_hrd;  // hardware address format
+   u_int16_t arp_pro;  // protocol address format 
+   u_int8_t  arp_hln;  // hardware address length
+   u_int8_t  arp_pln;  // protocol address length
+   u_int16_t arp_op;   // ARP operation type
+   u_int8_t  arp_smac[6];  // sender's hardware address
+   u_int8_t  arp_sip[4];  // sender's protocol address
+   u_int8_t  arp_tmac[6];  // target hardware address
+   u_int8_t  arp_tip[4];  // target protocol address
+};
+
+
+
+//#pragma pack(1)
 struct struct_ip
 {
    u_int8_t 
@@ -110,28 +192,34 @@ struct struct_ip
    u_int8_t  dst[4];
 };
 
-struct struct_udp
-{
-  u_int16_t 
-     sp,
-     dp,
-     len,
-     sum;
+//#pragma pack(1)
+struct struct_udp {
+	u_int16_t 
+		sp,
+		dp,
+		len,
+		sum;
 };
 
-struct struct_rtp
-{
-  u_int8_t 
-     byte1,
-     ptype;
-   u_int16_t
-     sqnr;
-   u_int32_t
-     timestamp,  // official timestamp, created by codecs
-     ssrc,
-//   csrc,      // only used by mixers
-     time_sec,
-     time_usec; // already within payload, microseconds
+//#pragma pack(1)
+struct struct_rtp {
+	u_int8_t 
+		byte1,
+		ptype;
+	u_int16_t
+		sqnr;
+	u_int32_t
+		timestamp,  // official timestamp, created by codecs
+		ssrc;
+        	//   csrc,      // only used by mixers
+	u_int16_t
+		ext_id,
+		ext_len;
+	u_int32_t
+		time_sec,
+		time_nsec,
+		time_sec2,
+		time_nsec2;
 };
 
 // ---------End of PCAP-specific definitions---------------
@@ -158,13 +246,14 @@ enum operating_modes
      CDP,
      RTP,
      RX_RTP,
-     SYSLOG
+     SYSLOG,
+     LLDP
 } mode;
 
 
 int quiet;           // don't even print 'important standard short messages'
 int verbose;         // report character
-int monotonic_clock; // chooses clock, see time.c
+int simulate;        // if 1 then don't really send frames
 
 char path[256];
 char filename[256];
@@ -173,44 +262,61 @@ FILE *fp, *fp2;             // global multipurpose file pointer
 long double total_d;
 clock_t mz_start, mz_stop;
 
-struct timeval 
-  tv,                // keeps sec and usec
-  timeTX[TIME_COUNT_MAX],
-  timeRX[TIME_COUNT_MAX];
-long int
+enum rtp_display_mode {
+	BAR, NCURSES, TEXT
+} rtp_dm;
+	
+
+int mz_rand;
+int bwidth;
+
+struct mz_timestamp {
+	u_int32_t sec; 
+	u_int32_t nsec;
+};
+
+struct mz_timestamp 
+	tv, 
+	timeTX[TIME_COUNT_MAX],  
+	timeRX[TIME_COUNT_MAX];
+
+int32_t
   time0,
   jitter_rfc,
-  jitter[TIME_COUNT_MAX];      // in usec
+  jitter[TIME_COUNT_MAX];   
 
 int 
   rtp_log,
   time0_flag,        // If set then time0 has valid data
   sqnr0_flag;  
 
+u_int8_t
+  mz_ssrc[4];     // holds RTP stream identifier for rcv_rtp()
+
 u_int16_t 
   sqnr_cur,
   sqnr_last, 
-  sqnr_next,
+  sqnr_next;
+
+u_int32_t
   drop,    // packet drop count
-  dis;     // packet disorder count
-
-
-u_int32_t 
+  dis,     // packet disorder count
   gind,      // a global index to run through deltaRX, deltaTX, and jitter
   gind_max,  // the amount of entries used in the (ugly oversized) arrays; per default set to TIME_COUNT
   gtotal;    // counts number of file write cycles (see "got_rtp_packet()") 
 
+
 char rtp_filter_str[64];
 
 struct tx_struct
 {
    // Management issues for TX
-   char device[16];
+   char device[16];           // every packet could be sent through a different device
    int  packet_mode;          // 0 means use LIBNET_LINK_ADV, 1 means LIBNET_RAW4
-   unsigned int count;        // 0 means inifite, 1 is default
-   unsigned int delay;        // POSIX conform delay in seconds, 0 means no delay (default)
+   unsigned int count;        // 0 means infinite, 1 is default
+   unsigned int delay;        // Delay in microseconds, 0 means no delay (default)
    char arg_string[MAX_PAYLOAD_SIZE];  // Argument-string when -t is used
-
+   
    // Ethernet and 802.3 parameters
    int eth_params_already_set; // if set to 1 then send_eth should only send the frame
    u_int8_t  eth_mac_own[6];  // Contains own interface MAC if needed by some modules
@@ -251,6 +357,10 @@ struct tx_struct
    int       ascii;           // 1 if specified
    u_int8_t  ascii_payload[MAX_PAYLOAD_SIZE];
 
+   // HEX PAYLOAD
+   u_int8_t  hex_payload[MAX_PAYLOAD_SIZE];
+   u_int32_t hex_payload_s;   // >0 if hex payload is specified
+   
    // MPLS Parameters
    char      mpls_txt[128];   // contains MPLS parameters (label, exp, S, TTL)
    char      mpls_verbose_string[1024]; // contains all labels for print_frame_details()
@@ -347,7 +457,11 @@ struct tx_struct
      rtp_sqnr,
      rtp_stmp;
    
-} tx;
+} tx;  // NOTE: tx elements are considered as default values for MOPS
+
+
+
+
 
 u_int8_t  gbuf[MAX_PAYLOAD_SIZE];  // This is only a generic global buffer to handover data more easily
 u_int32_t gbuf_s;                  //
@@ -364,8 +478,20 @@ int reset();
 void usage();
 int getopts(int argc, char *argv[]);
 int getarg(char *str, char *arg_name, char *arg_value);
-unsigned long int str2int(char *str);
-  
+unsigned long int str2int(char *str);       // converts "65535" to 65535
+unsigned long long int str2lint(char *str); // same but allows 64-bit integers
+unsigned long int xstr2int(char *str);      // converts "ffff" to 65535
+unsigned long long int xstr2lint(char *str);     // same but allows 64-bit integers
+int mz_strisbinary(char *str);
+int mz_strisnum(char *str);
+int mz_strishex(char *str);
+int str2bin8 (char *str);
+long int str2bin16 (char *str);
+int char2bits (char c, char *str);
+int mz_strcmp(char* usr, char* str, int min);
+int mz_tok(char * str, char * delim, int anz, ...);
+int delay_parse (struct timespec *t, char *a, char *b);
+
 // ************************************
 // 
 //  Prototypes: Layer1
@@ -428,13 +554,13 @@ int create_syslog_packet();
 
 // Converts MAC address specified in str into u_int8_t array
 // Usage: str2hex_mac ( "00:01:02:aa:ff:ee", src_addr )
-void str2hex_mac (char* str, u_int8_t *addr);
+int str2hex_mac (char* str, u_int8_t *addr);
 
 // Converts ascii hex values (string) into integer array, similarly as above but for any size.
 // Example: "1a 00:00-2f" => {26, 0, 0, 47}
 // Note: apply any improvements here and prefer this function in future!
 // Return value: Number of converted elements (=length of array)
-int str2hex (char* str, u_int8_t *hp);
+int str2hex (char* str, u_int8_t *hp, int n);
 
 // Converts ascii numbers (string) into integer array
 // Every byte can be specified as integers {0..255}
@@ -519,6 +645,9 @@ int check_eth_mac_txt(int src_or_dst);
 // 
 int get_port_range (int sp_or_dp, char *arg);
 
+// Return a 4-byte unsigned int random number
+u_int32_t  mz_rand32 ();
+
 // Scans argument for TCP flags and sets 
 // tx.tcp_control accordingly.
 // 
@@ -625,12 +754,15 @@ int complexity();
 
 
 // Purpose: Calculate time deltas of two timestamps stored in struct timeval.
-// This function is a slightly modified version of an example found in the glibc book
 // Subtract the "struct timeval" values X and Y, storing the result in RESULT.
 // Return 1 if the difference is negative, otherwise 0.
-int timeval_subtract (struct timeval *x,
-		      struct timeval *y,
-		      struct timeval *result);
+int timestamp_subtract (struct mz_timestamp *x,
+			struct mz_timestamp *y, 
+			struct mz_timestamp *result);
+
+void timestamp_add (struct mz_timestamp *x, 
+		    struct mz_timestamp *y, 
+		    struct mz_timestamp *result);
 
 // Returns a human readable timestamp in the string result.
 // Optionally a prefix can be specified, for example if the
@@ -669,6 +801,9 @@ int rcv_rtp_init();
 // Defines the pcap handler and the callback function
 int rcv_rtp();
 
+// Print current RFC-Jitter on screen
+void print_jitterbar (long int j, unsigned int d);
+
 // Compares two 4-byte variables byte by byte
 // returns 0 if identical, 1 if different
 int compare4B (u_int8_t *ip1, u_int8_t *ip2);
@@ -687,6 +822,16 @@ int compare4B (u_int8_t *ip1, u_int8_t *ip2);
 //  
 int lookupdev();
 
+
+// For a given device name, find out the following parameters:
+// 
+//  - MTU
+//  - Network
+//  - Mask
+//  - Default GW (IP)
+//  
+int get_dev_params (char *name);
+
 // Handler function to do something when RTP messages are received
 void got_rtp_packet(u_char *args,
 		    const struct pcap_pkthdr *header, // statistics about the packet (see 'struct pcap_pkthdr')
@@ -704,9 +849,47 @@ int check_timer();
 // measurement would include wrong datapoints.
 // 
 // Furthermore the function below utilizes the newer hi-res nanosecond timers.
-// However it is rounded to microseconds so all other functions may use the
-// traditional struct timeval (which consists of sec and usec).
-// 
-int getcurtime (struct timeval *tv);
+inline void getcurtime (struct mz_timestamp *t);
+
+// Only print out the help text for the 02.1Q option
+void print_dot1Q_help(void);
+
+// Determines ip and mac address of specified interface 'ifname'
+// Caller must provide an unsigned char ip[4], mac[6]
+//
+int get_if_addr (char *ifname, unsigned char *ip, unsigned char *mac);
+
+// Takes filename and prepends valid configuration/logging directory
+// NOTE: filename is overwritten and must be big enough to hold full path!
+int getfullpath_cfg (char *filename);
+int getfullpath_log (char *filename);
+
+// A safer replacement for strncpy which ensures \0-termination
+char * mz_strncpy(char *dest, const char *src, size_t n);
+
+// Helper function to count the number of arguments
+// in the Mausezahn argument string (comma separated args)
+// RETURN VALUE: Number of arguments
+int number_of_args (char *str);
+
+int arptable_add(struct device_struct *dev, 
+		 u_int8_t *sa, 
+		 u_int8_t *da, 
+		 u_int8_t *smac, 
+		 u_int8_t *sip, 
+		 u_int32_t sec, 
+		 u_int32_t nsec);
+
+// Validate ARP requests
+int arpwatch(struct device_struct *dev, 
+	     u_int8_t *sa, 
+	     u_int8_t *da, 
+	     u_int8_t *smac, 
+	     u_int8_t *sip, 
+	     u_int8_t *tmac,
+	     u_int8_t *tip,
+	     u_int32_t sec,
+	     u_int32_t nsec);
+
 
 #endif
