@@ -1,6 +1,6 @@
 /*
  * Mausezahn - A fast versatile traffic generator
- * Copyright (C) 2008 Herbert Haas
+ * Copyright (C) 2008,2009 Herbert Haas
  * 
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License version 2 as published by the 
@@ -18,6 +18,36 @@
 
 
 #include "mz.h"
+#include "cli.h"
+
+#define MZ_SYSLOG_HELP \
+   		"| Syslog type: Send (traditional) Syslog packets via UDP.\n" \
+		"|\n" \
+		"| Parameters:\n" \
+		"|\n" \
+		"|  severity, sev  0-7            .... Severity level from Emergency (0) to Debug (7)\n" \
+		"|  facility, fac  0-23           .... Facility number\n" \
+		"|\n" \
+		"|  time           hh:mm:ss       .... Local time, 24-hour format\n" \
+		"|  month, mon     Mmm            .... Current month, 1-12\n" \
+		"|  day            dd             .... Current day, 0-31\n" \
+		"|\n" \
+		"|  host           max 314 bytes  .... Name or IP Address of sending host\n" \
+		"|\n" \
+		"| Defaults:\n" \
+		"|\n" \
+		"|  Per default the severity \"Warning\" (4), the facility \"Security\" (4), and the\n" \
+		"|  current time stamp is used. If no host is given, host is set to \"MZ\"\n"  \
+		"|\n" \
+		"| You can define the Syslog message itself using the -P flag. For example:\n" \
+		"|\n" \
+		"|   mz eth0 -t syslog sev=3 -P \"You have been mausezahned.\"\n" \
+		"|\n" \
+		"| By the way, mz (by intention) does not check if your timestamp is valid according\n" \
+                "| calendar rules. It is generally recommended to follow the Darwin Era Calendar ;-)\n" \
+		"|\n"
+
+
 
 // RFC 3164 states that a Syslog message consists of three parts: PRI, HEADER, and MSG.
 // 
@@ -35,75 +65,66 @@
 //  
 //      <34>Oct 11 22:14:15 mymachine su: 'su root' failed for lonvick on /dev/pts/8
 //      
-//          
+//  EXAMPLE from Cisco Router:
+//        
+//      *Mar 23 13:45:08.727: %ENVMON-3-FAN_FAILED: Fan 2 not rotating
+//                  
 
 
 int create_syslog_packet()
 {
-   unsigned int pri, sev, fac, day, curday, mon, curmon;
-   char  lt[8], host[314];
-   char *Months[12] = 
-     { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"  };
-   
-
-   time_t curtime;
-   struct tm curtime_broken;
-   
-   
-   char argval[MAX_PAYLOAD_SIZE];
-   
-   if ( (getarg(tx.arg_string,"help", NULL)==1) && (mode==SYSLOG) )
-     {
-	fprintf(stderr,"\n"
-		MAUSEZAHN_VERSION
-		"\n"
-		"| Syslog type: Send (traditional) Syslog packets via UDP.\n"
-		"|\n"
-		"| Parameters:\n"
-		"|\n"
-		"|  severity, sev  0-7            .... Severity level from Emergency (0) to Debug (7)\n"
-		"|  facility, fac  0-23           .... Facility number\n"
-		"|\n"
-		"|  time           hh:mm:ss       .... Local time, 24-hour format\n"
-		"|  month, mon     Mmm            .... Current month, 1-12\n"
-		"|  day            dd             .... Current day, 0-31\n"
-		"|\n"
-		"|  host           max 314 bytes  .... Name or IP Address of sending host\n"
-		"|\n"
-		"| Defaults:\n"
-		"|\n"
-		"|  Per default the severity \"Warning\" (4), the facility \"Security\" (4), and the\n"
-		"|  current time stamp is used. If no host is given, host is set to \"MZ\"\n" 
-		"|\n"
-		"| You can define the Syslog message itself using the -P flag. For example:\n"
-		"|\n"
-		"|   mz eth0 -t syslog sev=3 -P \"Sie wurden soeben gemausezahnt.\"\n"
-		"|\n"
-		"| By the way, I do not check if your timestamp is valid according calendar rules ;-)\n"
-		"|\n"
-		);
-	exit(0);
-     }
+	unsigned int pri, sev, fac, day, curday, mon, curmon;
+	char  lt[8], host[314];
+	char *Months[12] = 
+	{ "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"  };
+	
+	
+	time_t curtime;
+	struct tm curtime_broken;
+	char argval[MAX_PAYLOAD_SIZE];
+	int ca=0, aa;
+	
+	aa=number_of_args(tx.arg_string);
+	
+	if ( (getarg(tx.arg_string,"help", NULL)==1) && (mode==SYSLOG) )
+	{
+		ca++; // counts each argument
+		if (mz_port)
+		{
+			cli_print(gcli, "%s", MZ_SYSLOG_HELP);
+			return -1;
+		}
+		else
+		{
+			fprintf(stderr,"\n" 
+				MAUSEZAHN_VERSION
+				"\n%s", MZ_SYSLOG_HELP);
+			
+			exit(0);
+		}
+	}
    
    
    if ( (getarg(tx.arg_string,"severity", argval)==1) || 
 	(getarg(tx.arg_string,"sev", argval)==1) )
      {
-	sev = (unsigned int) str2int(argval);
+	     ca++; // counts each argument
+	     sev = (unsigned int) str2int(argval);
      }
    else
-     {
-	sev = 4;
+     {		
+	     sev = 4;
      }
 
    if ( (getarg(tx.arg_string,"facility", argval)==1) || 
 	(getarg(tx.arg_string,"fac", argval)==1) )
      {
-	fac = (unsigned int) str2int(argval);
+	     ca++; // counts each argument
+	     fac = (unsigned int) str2int(argval);
      }
    else
      {
-	fac = 4;
+	     fac = 4;
      }
 
    
@@ -114,12 +135,13 @@ int create_syslog_packet()
    
    if (getarg(tx.arg_string,"time", argval)==1)
      {
-	strncpy(lt,argval,8);
-	// TODO: check if specified timestamp has valid format, e. g. 15:03:22
+	     ca++; // counts each argument
+	     strncpy(lt,argval,8);
+	     // TODO: check if specified timestamp has valid format, e. g. 15:03:22
      }
    else
      {
-	timestamp_hms (lt);
+	     timestamp_hms (lt);
      }
 
 
@@ -129,6 +151,7 @@ int create_syslog_packet()
    if ( (getarg(tx.arg_string,"month", argval)==1) || 
 	(getarg(tx.arg_string,"mon", argval)==1) )
      {
+	ca++; // counts each argument
 	mon = (unsigned int) str2int(argval);
 	if ( (mon<1) || (mon>12) )
 	  {
@@ -145,6 +168,7 @@ int create_syslog_packet()
    
    if (getarg(tx.arg_string,"day", argval)==1) 
      {
+        ca++; // counts each argument
 	day = (unsigned int) str2int(argval);
 	if ( (day<1) || (day>31) )
 	  {
@@ -160,6 +184,7 @@ int create_syslog_packet()
 
    if (getarg(tx.arg_string,"host", argval)==1)
      {
+        ca++; // counts each argument
 	strncpy(host,argval,314);  // 314 is just an arbitrary number ;-)
      }
    else
@@ -167,7 +192,13 @@ int create_syslog_packet()
 	strcpy(host, "MZ42"); 
      }
 
-   
+  
+   // CHECK SURPLUS ARGUMENTS
+   if (aa!=ca) {
+	   fprintf(stderr, "WARNING: %i unmatched arguments within argument string!\n", aa-ca);
+   }
+	
+	
    // Now put everything together:
    // 
    //  Again the EXAMPLE from RFC 3164:
